#proxymin - A Web-based administration frontend for the Squid Web proxy cache
#Copyright (C) 2004  Mirjam Kuhlmann (proxymin@mikuhl.de)
#Copyright (C) 2004  Clemens Hermann (proxymin@clhe.de)
#
#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

import string
import os

import ConfigParser

from globals import *
import dbaccess
import permissions

config = ConfigParser.ConfigParser()
config.add_section("files")
config.read(CONFIG_FILE_PATH + CONFIG_FILE)

# if the dbase-files do not exist they are created automatically
squid_cfg = config.get("files","squid_cfg")
static_cfg = config.get("files","static_cfg")
squid_reconfigure_command = config.get("commands","squid_reconfigure_command")
case_sensitive = (config.get("parameters","case_sensitive") == 'TRUE')

def create_acl(id, category, urls, protocol, permission=NONE):
    if category == CATEGORY_GROUPS:
        object_string = 'group_' + id     
    elif category == CATEGORY_HOSTS:
        object_string = 'host_' + string.replace(id,'.','_')
    else:
        object_string = 'user_' + id
    result = 'acl ' + object_string + '_'
    if protocol == PROTOCOL_HTTP:
        result += 'http'
    else:
        result += 'ftp'
    if permission == ALLOWED:
        result += '_allowed'
    elif permission == DENIED:
        result += '_denied'
    result += ' url_regex'
    if not case_sensitive:
        result += ' -i'
    for url in urls:
        result += ' ^'
        if protocol == PROTOCOL_HTTP:
            result += '(http://)?'
        else:
            result += 'ftp://'
        result += url
    return result + '\n'

def is_allowed_urls_empty(object, protocol):
    if protocol == PROTOCOL_HTTP:
        urls = object.get_http_url_list()
    else:
        urls = object.get_ftp_url_list()
    for url in urls:
        if protocol == PROTOCOL_HTTP:
            if object.is_http_url_allowed(url):
                return FALSE
        else:
            if object.is_ftp_url_allowed(url):
                return FALSE
    return TRUE

def is_denied_urls_empty(object, protocol):
    if protocol == PROTOCOL_HTTP:
        urls = object.get_http_url_list()
    else:
        urls = object.get_ftp_url_list()
    for url in urls:
        if protocol == PROTOCOL_HTTP:
            if not object.is_http_url_allowed(url):
                return FALSE
        else:
            if not object.is_ftp_url_allowed(url):
                return FALSE
    return TRUE

def is_ssl_enabled(object):
    if object.is_ssl_enabled():
        return TRUE
    elif object.is_ssl_disabled():
        return FALSE
    else:
        a = dbaccess.DBAccess()
        group = a.get_group(object.get_group())
        if group.get_http_permissions().is_ssl_enabled():
            return TRUE
        else:
            return FALSE    

def create_http_access(id, category):
    a = dbaccess.DBAccess()
    if category == CATEGORY_HOSTS:
        object = a.get_host(id)
        object_string = 'host_' + string.replace(id,'.','_')
    else:
        object = a.get_user(id)
        object_string = 'user_' + id
    group = a.get_group(object.get_group())
    result = ''
    if object.is_enabled():
        # denied extensions
        if len(group.get_denied_extensions_list()) > 0:
            result += 'http_access deny ' + object_string + ' group_' + object.get_group() + '_extensions\n'
        # denied URLs for user/host
        if not is_denied_urls_empty(object, PROTOCOL_HTTP):
            result += 'http_access deny ' + object_string + ' HTTP http_ports ' + object_string + '_http_denied\n'
            if is_ssl_enabled(object):
                result += 'http_access deny ' + object_string + ' CONNECT SSL_ports ' + object_string + '_http_denied\n'
        if not is_denied_urls_empty(object, PROTOCOL_FTP):
            result += 'http_access deny ' + object_string + ' FTP ftp_ports ' + object_string + '_ftp_denied\n'
        # allowed URLs for user/host
        if not is_allowed_urls_empty(object, PROTOCOL_HTTP):
            result += 'http_access allow ' + object_string + ' HTTP http_ports ' + object_string + '_http_allowed\n'
            if is_ssl_enabled(object):
                result += 'http_access allow ' + object_string + ' CONNECT SSL_ports ' + object_string + '_http_allowed\n'
        if not is_allowed_urls_empty(object, PROTOCOL_FTP):
            result += 'http_access allow ' + object_string + ' FTP ftp_ports ' + object_string + '_ftp_allowed\n'
        # URLs for group
        if(len(group.get_http_permissions().get_url_list()) > 0 or group.get_http_permissions().is_default_allow()):
            result += 'http_access allow ' + object_string + ' HTTP http_ports'
            if len(group.get_http_permissions().get_url_list()) > 0:
                if group.get_http_permissions().is_default_allow():
                    result += ' !'
                else:
                    result += ' '
                result += 'group_' + object.get_group() + '_http'
            result += '\n'
            if is_ssl_enabled(object):    
                result += 'http_access allow ' + object_string + ' CONNECT SSL_ports'
                if len(group.get_http_permissions().get_url_list()) > 0:
                    if group.get_http_permissions().is_default_allow():
                        result += ' !'
                    else:
                        result += ' '
                    result += 'group_' + object.get_group() + '_http'
                result += '\n'
        if(len(group.get_ftp_permissions().get_url_list()) > 0 or group.get_ftp_permissions().is_default_allow()):
            result += 'http_access allow ' + object_string + ' FTP ftp_ports'
            if len(group.get_ftp_permissions().get_url_list()) > 0:
                if group.get_ftp_permissions().is_default_allow():
                    result += ' !'
                else:
                    result += ' '
                result += 'group_' + object.get_group() + '_ftp'
            result += '\n'
    result += 'http_access deny ' + object_string + ' all\n'
    return result
     
def update():
    """reads the database content and generates the squid configuration"""
    a = dbaccess.DBAccess()
    acl_users = '' 
    acl_hosts = ''
    acl_group_rules = ''
    acl_user_rules = '' 
    acl_host_rules = ''
    http_access = ''
    acl_access = 'acl all src 0.0.0.0/0.0.0.0\n'
    acl_access += 'acl ftp_ports port 21\n'
    acl_access += 'acl http_ports port 80\n'
    acl_access += 'acl SSL_ports port 443\n'
    acl_access += 'acl CONNECT method CONNECT\n'
    acl_access += 'acl FTP proto FTP\n'    
    acl_access += 'acl HTTP proto HTTP\n'

    for group_id in a.get_group_list():
        group = a.get_group(group_id)
        urls = []
        for url in group.get_http_permissions().get_url_list():
            urls.append(url)
        if len(urls) > 0:
            acl_group_rules += create_acl(group_id, CATEGORY_GROUPS, urls, PROTOCOL_HTTP)
        urls = []
        for url in group.get_ftp_permissions().get_url_list():
            urls.append(url)
        if len(urls) > 0:
            acl_group_rules += create_acl(group_id, CATEGORY_GROUPS, urls, PROTOCOL_FTP)
        # acl for extensions
        if len(group.get_denied_extensions_list()) > 0:
            acl_group_rules += 'acl group_' + group_id + '_extensions url_regex'
            if not case_sensitive:
                acl_group_rules += ' -i'
            for extension in group.get_denied_extensions_list():
                acl_group_rules += ' ' + extension + '$'
            acl_group_rules += '\n'

    for ip in a.get_host_list():
        host = a.get_host(ip)
        acl_hosts += 'acl host_' + string.replace(ip,'.','_') + ' src ' + ip + '/255.255.255.255\n'        
        if host.is_enabled():
            urls_allowed = []
            urls_denied = []
            for url in host.get_http_url_list():
                if host.is_http_url_allowed(url):
                    urls_allowed.append(url)
                else:
                    urls_denied.append(url)
            if len(urls_denied) > 0:
                acl_host_rules += create_acl(ip, CATEGORY_HOSTS, urls_denied, PROTOCOL_HTTP, DENIED)
            if len(urls_allowed) > 0:
                acl_host_rules += create_acl(ip, CATEGORY_HOSTS, urls_allowed, PROTOCOL_HTTP, ALLOWED)
            urls_allowed = []
            urls_denied = []
            for url in host.get_ftp_url_list():
                if host.is_ftp_url_allowed(url):
                    urls_allowed.append(url)
                else:
                    urls_denied.append(url)
            if len(urls_denied) > 0:
                acl_host_rules += create_acl(ip, CATEGORY_HOSTS, urls_denied, PROTOCOL_FTP, DENIED)
            if len(urls_allowed) > 0:
                acl_host_rules += create_acl(ip, CATEGORY_HOSTS, urls_allowed, PROTOCOL_FTP, ALLOWED)
        http_access += create_http_access(ip, CATEGORY_HOSTS) + '\n'

    for uid in a.get_user_list():
        user = a.get_user(uid)
        acl_users += 'acl user_' + uid + ' proxy_auth ' + uid + '\n'        
        if user.is_enabled():
            urls_allowed = []
            urls_denied = []
            for url in user.get_http_url_list():
                if user.is_http_url_allowed(url):
                    urls_allowed.append(url)
                else:
                    urls_denied.append(url)
            if len(urls_denied) > 0:
                acl_user_rules += create_acl(uid, CATEGORY_USERS, urls_denied, PROTOCOL_HTTP, DENIED)
            if len(urls_allowed) > 0:
                acl_user_rules += create_acl(uid, CATEGORY_USERS, urls_allowed, PROTOCOL_HTTP, ALLOWED)
            urls_allowed = []
            urls_denied = []
            for url in user.get_ftp_url_list():
                if user.is_ftp_url_allowed(url):
                    urls_allowed.append(url)
                else:
                    urls_denied.append(url)
            if len(urls_denied) > 0:
                acl_user_rules += create_acl(uid, CATEGORY_USERS, urls_denied, PROTOCOL_FTP, DENIED)
            if len(urls_allowed) > 0:
                acl_user_rules += create_acl(uid, CATEGORY_USERS, urls_allowed, PROTOCOL_FTP, ALLOWED)
        http_access += create_http_access(uid, CATEGORY_USERS) + '\n'

    cfg_file = open(squid_cfg, 'w')
    static_file = open(static_cfg, 'r')
   
    cfg = static_file.readlines()
    cfg += '\n'
    cfg += acl_access + '\n'
    if acl_users != '':
        cfg += '#\n# users\n#\n' 
        cfg += acl_users + '\n' 
    if acl_hosts != '':
        cfg += '#\n# hosts\n#\n' 
        cfg += acl_hosts + '\n'
    if acl_group_rules != '':        
        cfg += '#\n# group rules\n#\n'     
        cfg += acl_group_rules + '\n'
    if acl_host_rules != '':     
        cfg += '#\n# host rules\n#\n'     
        cfg += acl_host_rules + '\n'
    if acl_user_rules != '':   
        cfg += '#\n# user rules\n#\n'    
        cfg += acl_user_rules + '\n' 
    cfg += '#\n# access rules\n#\n'     
    cfg += http_access
    cfg += 'http_access deny all\n'

    cfg_file.writelines(cfg)
    cfg_file.close()
    static_file.close()
    os.system(squid_reconfigure_command)
